#include "lsqrms.h"

#if HAVE_MEMORY_H
# include <memory.h>
#endif

#include "provepdb.h"
#include "matvecutils.h"

char *provepdb (char *filename, int altered, int use_carbonyl) {
  FILE *file_stream;
  char *return_string = NULL;
  int i = 0, j = 0;
  int saw_ter = 0;
  char c;
  char *line_read = NULL;
  int line_length = 0;
  int abc = 0;
  char *new_string = NULL;
  char *chem = NULL;
  char *which = NULL;
  double x, y, z;
  char *tempstr = NULL;

  file_stream = fopen(filename, "r");

  if (file_stream == NULL)
    return NULL;

  tempstr = malloc(sizeof (char) * 9);
  if (!tempstr) {
    printf("provepdb: Allocation failed!\n");
    return NULL;
  }
  tempstr[8] = (char)NULL;

  do {
    abc++;
    do {
      if ((fread(&c, sizeof (char), 1, file_stream) == 1) &&
	  (!feof(file_stream))) {
	line_read = (char *)realloc(line_read, sizeof(char)*(line_length + 2));
	line_read[line_length] = c;
	line_read[line_length + 1] = (char) 0;
	line_length = strlen(line_read);
      } else
	break;
    } while (c != (char) '\n');

    if ((line_length >= 6) &&
	(strncmp(line_read, "ENDMDL", 6) == 0))
      break;

    if (line_length >= 50) {
      chem = malloc(sizeof (char) * 4);
      if (!chem) {
	printf("provepdb: Allocation failed!\n");
	return NULL;
      }
      which = malloc(sizeof (char) * 2);
      if (!chem) {
	printf("provepdb: Allocation failed!\n");
	return NULL;
      }

      if (sscanf(line_read, "ATOM %*d CA%*[ ] %3[^ ]%*[ ]",
		 chem) == 1) {
	new_string = malloc(sizeof(char) * (strlen(line_read) + 1));
	if (!new_string) {
	  fprintf(stderr,"provepdb: Allocation failed!\n");
	  return NULL;
	}
	
	i++;
	j++;
	
	line_read[29] = '0';
	line_read[37] = '0';
	line_read[45] = '0';
	tempstr = memcpy(tempstr, (char *)(line_read + 30), 8);
	x = strtod(tempstr, NULL);
	tempstr = memcpy(tempstr, (char *)(line_read + 38), 8);
	y = strtod(tempstr, NULL);
	tempstr = memcpy(tempstr, (char *)(line_read + 46), 8);
	z = strtod(tempstr, NULL);

	sprintf(new_string,
		"ATOM%7d  CA   %.3s %4d %12.3f %8.3f %8.3f\n",
		i, chem, j, x, y, z);
	new_string = (char *)realloc(new_string, strlen (new_string) + 1);
	if (return_string == NULL)
	  return_string = strdup("");
	return_string = (char *)realloc(return_string,
					sizeof(char)*(strlen(return_string) + 
						      strlen(new_string) + 1));
	return_string = strcat(return_string, new_string);
	free(new_string);
	new_string = NULL;
      } else if ((!saw_ter) &&
		 (sscanf(line_read, "HETATM%*d CA%*[ ] %3[^ ]%*[ ]",
			 chem) == 1)) {
	if (convert321(chem, 0, altered) != '?') {
	  new_string = malloc(sizeof(char) * (strlen(line_read) + 1));
	  if (!new_string) {
	    fprintf(stderr,"provepdb: Allocation failed!\n");
	    return NULL;
	  }
	  
	  i++;
	  j++;
	  
	  line_read[29] = '0';
	  line_read[37] = '0';
	  line_read[45] = '0';
	  tempstr = memcpy(tempstr, (char *)(line_read + 30), 8);
	  x = strtod(tempstr, NULL);
	  tempstr = memcpy(tempstr, (char *)(line_read + 38), 8);
	  y = strtod(tempstr, NULL);
	  tempstr = memcpy(tempstr, (char *)(line_read + 46), 8);
	  z = strtod(tempstr, NULL);

	  sprintf(new_string,
		  "ATOM%7d  CA   %.3s %4d %12.3f %8.3f %8.3f\n",
		  i, chem, j, x, y, z);
	  new_string = (char *)realloc(new_string, strlen (new_string) + 1);
	  if (return_string == NULL)
	    return_string = strdup("");
	  return_string =
	    (char *)realloc(return_string,
			    sizeof(char)*(strlen(return_string) + 
					  strlen(new_string) + 1));
	  return_string = strcat(return_string, new_string);
	  free(new_string);
	  new_string = NULL;
	}
      } else if (use_carbonyl &&
		 (sscanf(line_read, "ATOM %*d %[OCN]%*[ ] %3[^ ]%*[ ]",
			 which, chem) == 2)) {
	new_string = malloc(sizeof(char) * (strlen(line_read) + 1));
	if (!new_string) {
	  fprintf(stderr,"provepdb: Allocation failed!\n");
	  return NULL;
	}
	
	i++;
	
	line_read[29] = '0';
	line_read[37] = '0';
	line_read[45] = '0';
	tempstr = memcpy(tempstr, (char *)(line_read + 30), 8);
	x = strtod(tempstr, NULL);
	tempstr = memcpy(tempstr, (char *)(line_read + 38), 8);
	y = strtod(tempstr, NULL);
	tempstr = memcpy(tempstr, (char *)(line_read + 46), 8);
	z = strtod(tempstr, NULL);

	if (which[0] == 'N')
	  sprintf(new_string,
		"ATOM%7d  %s    %.3s %4d %12.3f %8.3f %8.3f\n",
		i, which, chem, j+1, x, y, z);
	else
	  sprintf(new_string,
		  "ATOM%7d  %s    %.3s %4d %12.3f %8.3f %8.3f\n",
		  i, which, chem, j, x, y, z);
	new_string = (char *)realloc(new_string, strlen (new_string) + 1);
	if (return_string == NULL)
	  return_string = strdup("");
	return_string = (char *)realloc(return_string,
					sizeof(char)*(strlen(return_string) + 
						      strlen(new_string) + 1));
	return_string = strcat(return_string, new_string);
	free(new_string);
	new_string = NULL;
      } else if (use_carbonyl && (! saw_ter) &&
		 (sscanf(line_read, "HETATM%*d %[OCN]%*[ ] %3[^ ]%*[ ]",
			 which, chem)
		  == 2)) {
	if (convert321(chem, 0, altered) != '?') {
	  new_string = malloc(sizeof(char) * (strlen(line_read) + 1));
	  if (!new_string) {
	    fprintf(stderr,"provepdb: Allocation failed!\n");
	    return NULL;
	  }
	  
	  i++;
	  
	  line_read[29] = '0';
	  line_read[37] = '0';
	  line_read[45] = '0';
	  tempstr = memcpy(tempstr, (char *)(line_read + 30), 8);
	  x = strtod(tempstr, NULL);
	  tempstr = memcpy(tempstr, (char *)(line_read + 38), 8);
	  y = strtod(tempstr, NULL);
	  tempstr = memcpy(tempstr, (char *)(line_read + 46), 8);
	  z = strtod(tempstr, NULL);
	  
	  if (which[0] == 'N')
	    sprintf(new_string,
		    "ATOM%7d  %s    %.3s %4d %12.3f %8.3f %8.3f\n",
		    i, which, chem, j+1, x, y, z);
	  else
	    sprintf(new_string,
		    "ATOM%7d  %s    %.3s %4d %12.3f %8.3f %8.3f\n",
		    i, which, chem, j, x, y, z);
	  new_string = (char *)realloc(new_string, strlen (new_string) + 1);
	  if (return_string == NULL)
	    return_string = strdup("");
	  return_string =
	    (char *)realloc(return_string,
			    sizeof(char)*(strlen(return_string) + 
					  strlen(new_string) + 1));
	  return_string = strcat(return_string, new_string);
	  free(new_string);
	  new_string = NULL;
	}
      }
      free(chem);
    }
    if ((!saw_ter) && (line_length >= 3) &&
	(strncmp(line_read, "TER", 3) == 0))
      saw_ter = 1;

    free(line_read);
    line_read = NULL;
    line_length = 0;

    if (feof(file_stream))
      break;
  } while (1);

  fclose(file_stream);
  file_stream = NULL;
  return return_string;
}
